! *****************************************************************
!
! This file contains a Fortran 90 module for (a) checking that the 
! linear friction coefficient is appropriate, and (b) for comparing 
! the sizes of the first- and  second-order components.
!
! Copyright (C) 2010 David McInerney, University of Chicago
!
! *****************************************************************
!
!    This program is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    any later version.
!
!    This program is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with this program (see file "COPYING").  If not, see
!    <http://www.gnu.org/licenses/>.
!
! *****************************************************************

module check_module

! *****************************************************************

    use analytic, only : analytic_zeta1, analytic_u1, analytic_v1, &
         analytic_zeta2, analytic_u2, analytic_v2, analytic_u, analytic_v, &
         analytic_zeta, &
         ZZ_bar, ZZ, ZZ_hat, PP_bar, PP1, PP_hat, QQ_bar, QQ, QQ_hat
    use parameters, only : LL, BB, period, g, h0, alpha, sigma, &
         Cl, tau_0, rho, Cb
    use output_module, only : j1, k1, j2, k2, j3, k3, j4, k4, j5, k5, j6, k6
    use grid, only : x_at_zeta, y_at_zeta, x_at_u, y_at_u

  contains

    subroutine check
    
    implicit none

    double precision x, y, t, delta_x, delta_y, delta_t, &
         velocity_sum, max_abs_zeta1, max_abs_zeta2, &
         max_abs_vel1, max_abs_vel2, &
         sum_abs_zeta1, sum_abs_zeta2, &
         sum_abs_vel1, sum_abs_vel2
    integer j, k, n, JJ, KK, NN, number, n_max

    JJ = 28
    KK = 20
    NN = 100
!!$    JJ = 140
!!$    KK = 100
!!$    NN = 1000

! ***********************************************************************
! check to see if Eqn (4), i.e., Cl/Cb = ave sqrt(U^2+V^2), is satisfied.

    velocity_sum = 0d0
    number = 0

    do j=0,JJ
       x = dble(j)/dble(JJ)*LL
       do k=1,KK
          y = dble(k)/dble(KK)*BB
          do n=1,NN
             t = dble(n)/dble(NN)*period
             number = number + 1
             velocity_sum = velocity_sum &
                  + sqrt(analytic_u(x,y,t)**2+analytic_v(x,y,t)**2)
          end do
       end do
    end do

    print*
    print*,"-------------------------------------------------------"
    print*
    print*,"Relationship between linear and quadratic friction coefficients"
    print*,"The following values should be approximately equal:"
    print*,"average velocity magnitude =",velocity_sum/dble(number)
    print*,"Cl/Cb =",Cl/Cb
    print*

! ***********************************************************************
! Check the relative sizes of first- and second-order components

    max_abs_zeta1 = 0d0
    max_abs_zeta2 = 0d0
    max_abs_vel1 = 0d0
    max_abs_vel2 = 0d0
    sum_abs_zeta1 = 0d0
    sum_abs_zeta2 = 0d0
    sum_abs_vel1 = 0d0
    sum_abs_vel2 = 0d0
    number = 0

    do j=0,JJ
       x = dble(j)/dble(JJ)*LL
       do k=0,KK
          y = dble(k)/dble(KK)*BB
          do n=1,NN
             t = dble(n)/dble(NN)*period
             if (abs(analytic_zeta1(x,y,t)).gt.max_abs_zeta1) then
                max_abs_zeta1 = abs(analytic_zeta1(x,y,t))
             end if
             if (abs(analytic_zeta2(x,y,t)).gt.max_abs_zeta2) then
                max_abs_zeta2 = abs(analytic_zeta2(x,y,t))
             end if
             if (sqrt(analytic_u1(x,t)**2+analytic_v1(y,t)**2) &
                  .gt.max_abs_vel1) then
                max_abs_vel1 = sqrt(analytic_u1(x,t)**2+analytic_v1(y,t)**2)
             end if
            if (sqrt(analytic_u2(x,y,t)**2+analytic_v2(x,y,t)**2) &
                 .gt.max_abs_vel2) then
                max_abs_vel2 = sqrt(analytic_u2(x,y,t)**2 &
                     +analytic_v2(x,y,t)**2)
             end if
             sum_abs_zeta1 = sum_abs_zeta1 + abs(analytic_zeta1(x,y,t))
             sum_abs_zeta2 = sum_abs_zeta2 + abs(analytic_zeta2(x,y,t))
             sum_abs_vel1 = sum_abs_vel1 + sqrt(analytic_u1(x,t)**2 &
                  +analytic_v1(y,t)**2)
             sum_abs_vel2 = sum_abs_vel2 + sqrt(analytic_u2(x,y,t)**2 &
                  +analytic_v2(x,y,t)**2)
             number = number + 1
          end do
       end do
    end do

    print*,"-------------------------------------------------------"
    print*
    print*,"Check that zeta_1 is an order of magnitude smaller than h_0" 
    print*, "(max abs zeta 1)/h0 =",max_abs_zeta1/h0
    print*, "(ave abs zeta 1)/h0 =",sum_abs_zeta1/dble(number)/h0
    print*

    print*,"Check that zeta_2 is an order of magnitude smaller than zeta_1"
    print*, "(max abs zeta 2)/(max abs zeta1) =",max_abs_zeta2/max_abs_zeta1
    print*, "(ave abs zeta 2)/(ave abs zeta1) =",sum_abs_zeta2/sum_abs_zeta1
    print*

    print*,"Check that 2nd order velocity is an order of magnitude smaller than first order velocity"
    print*,"(max abs vel 2)/(max abs vel 1) =", max_abs_vel2/max_abs_vel1
    print*,"(ave abs vel 2)/(ave abs vel 1) =", sum_abs_vel2/sum_abs_vel1
    print*
    print*,"-------------------------------------------------------"

  end subroutine check

end module check_module
